@@ -13,7 +13,7 @@
 #########################################################################
 
 
-import re
+import re, random, inspect
 import socket
 
 import trio
@@ -72,12 +72,19 @@ class Color(object):
             self.text = "{}{}{}".format(self.esc, fg, text)
         self.str = self.text
 
+    @classmethod
+    def random(cls):
+        return random.choice(cls.COLORS)
+
     @classmethod
     def colors(cls):
+        """
+        Returns the color names
+        """
         return [
             k
             for k in Color.__dict__
-            if not (k.startswith("_") or k in ["esc", "COLORS", "colors"])
+            if not (k.startswith("_") or k in ["esc", "COLORS", "colors", "getcolors"])
         ]
 
     def __str__(self):
@@ -187,6 +194,13 @@ class persistentData(object):
 
 class Message(object):
     def __init__(self, channel="", sender_nick="", message="", is_private=False):
+        """Message
+
+        :param channel: Channel from/to which the message is sent or user/nick if private
+        :param sender_nick: Whoever's nick the message came from. Only for received messages. Aliases for this are nick.
+        :param message:str text of the message. Aliases: str, text, txt. For outgoing messages you can also set this to a Color object.
+        :param is_private: If True the message came from a user
+        """
         self.channel = channel.strip()
         self.sender_nick = sender_nick.strip()
         self.nick = sender_nick.strip()
@@ -254,6 +268,9 @@ class IrcBot(object):
 
         self.connected = False
 
+        if utils.arg_commands_with_message:
+            utils.setCommands(utils.arg_commands_with_message, prefix=utils.command_prefix)
+
         (
             self.send_message_channel,
             self.receive_message_channel,
@@ -572,14 +589,15 @@ class IrcBot(object):
                     channel in self.replyIntents
                     and sender_nick in self.replyIntents[channel]
                 ):
-                    result = self.replyIntents[channel][sender_nick].func(
+                    result = await self._call_cb(
+                        self.replyIntents[channel][sender_nick].func,
                         Message(channel, sender_nick, msg, is_private)
                     )
                     del self.replyIntents[channel][sender_nick]
                     await self.process_result(result, channel, sender_nick, is_private)
                     return
 
-                for i, cmd in enumerate(utils.regex_commands):
+                for i, cmd in enumerate(utils.regex_commands[::-1]):
                     if matched:
                         break
                     for reg in cmd:
@@ -589,7 +607,7 @@ class IrcBot(object):
                                 if is_private and not utils.regex_commands_accept_pm[i]:
                                     continue
                                 await trio.sleep(0)
-                                result = cmd[reg](m)
+                                result = await self._call_cb(cmd[reg], m)
                             if result:
                                 await self.process_result(
                                     result, channel, sender_nick, is_private
@@ -597,7 +615,7 @@ class IrcBot(object):
                                 matched = True
                                 continue
 
-                for i, cmd in enumerate(utils.regex_commands_with_message):
+                for i, cmd in enumerate(utils.regex_commands_with_message[::-1]):
                     if matched:
                         break
                     for reg in cmd:
@@ -614,13 +632,13 @@ class IrcBot(object):
                                 debug("sending to", sender_nick)
                                 if utils.regex_commands_with_message_pass_data[i]:
                                     await trio.sleep(0)
-                                    result = cmd[reg](
+                                    result = await self._call_cb(cmd[reg],
                                         m,
                                         Message(channel, sender_nick, msg, is_private),
                                     )
                                 else:
                                     await trio.sleep(0)
-                                    result = cmd[reg](
+                                    result = await self._call_cb(cmd[reg],
                                         m,
                                         Message(channel, sender_nick, msg, is_private),
                                     )
@@ -642,7 +660,7 @@ class IrcBot(object):
                     if utils.validateUrl(word):
                         await trio.sleep(0)
                         debug("Checking url: " + str(word))
-                        result = utils.url_commands[-1](word)
+                        result = await self._call_cb(utils.url_commands[-1], word)
                     if result:
                         await self.send_message(result, channel)
 
@@ -652,6 +670,11 @@ class IrcBot(object):
             log("ERROR IN MAINLOOP: ", e)
             logger.exception(e)
 
+    async def _call_cb(self, cb, *args, **kwargs):
+        if inspect.iscoroutinefunction(cb):
+            return await cb(*args, **kwargs)
+        return cb(*args, **kwargs)
+
     def __del__(self):
         try:
             self.s.close()
